﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace vcsharp_i2c_demo_v1._0
{
    public partial class I2CDemo : Form
    {
        IntPtr i2cHandle = IntPtr.Zero;                                                         ///< Handle to I2C
        uint returnValue = 0;                                                                   ///< Error checking variable
        Byte i2cEepromAddress = 80;                                                             ///< EEPROM Device address 
        Byte i2cRTCAddress = 104;                                                               ///< RTC Device address                                     
        Byte[] i2cEepromDataAddress = new Byte[2] {00, 00};                                     ///< First two bytes denote the EEPROM address where data will be written
        uint bufferLenght = 0;                                               
        const int eepromAddressWriteOffset = 2;

        public I2CDemo()
        {
            InitializeComponent();
            i2cHandle = i2c.I2c_Init("I2C1");                                                   ///< get handle to I2C device
            if (i2cHandle == IntPtr.Zero)
            {
                MessageBox.Show("Error in I2c_Init()");
            }
            if (!i2c.I2c_Open(i2cHandle))                                                       ///< open I2c for communication
            {
                MessageBox.Show("Error: I2C Open");
            }
            i2c.I2c_SetConfigInt(i2cHandle, "BitRateHz", 400000, TdxCommon.ParamStorageType.StoreVolatile); ///< Configuring I2C channel with appropriate settings
            i2c.I2c_SetConfigInt(i2cHandle, "SlaveAddrSize", 7, TdxCommon.ParamStorageType.StoreVolatile);  
            i2c.I2c_SetConfigInt(i2cHandle, "RegisterAddrSize", 0, TdxCommon.ParamStorageType.StoreVolatile);
            btnRTCRead.Enabled = false;                                                         ///< GUI Code
            rdbtnManual.Checked = false;
            tmrRTCRead.Enabled = true;
            tmrRTCRead.Interval = 100;
        }
        /// <summary>
        /// Event for Radio button to selct aotumatic and manual read of RTC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AutoManualEvent(object sender, EventArgs e)
        {
            if (rdbtnManual.Checked == true)                                                    ///< If manual selected, disable automatic radio button and timer 
            {
                btnRTCRead.Enabled = true;
                rdbtnAuto.Checked = false;
                tmrRTCRead.Enabled = false;
            }
            else
            {
                btnRTCRead.Enabled = false;                                                     ///< if manaul not selcted, enable timer to read RTC on given interval and disable read button
                rdbtnManual.Checked = false;
                tmrRTCRead.Enabled = true;
            }
        }
        /// <summary>
        /// Write to EEPROM starting from Address Ox00
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EepromWrite(object sender, EventArgs e)
        {
            btnRTCRead.Enabled = true;                                                          ///< GUI code
            rdbtnAuto.Checked = false;
            tmrRTCRead.Enabled = false;
            rdbtnManual.Checked = true;
            lblMessage.Text = " ";

            bufferLenght = (uint)txtEepromWrite.Text.Length;                                    ///< Get lenght of text entered into textbox
            Byte[] i2cEepromWrite = new Byte[bufferLenght + eepromAddressWriteOffset+1];        ///< Make buffer for writing to eeprom chip
            Byte[] temp = new Byte[bufferLenght];                                               ///< Temporary buffer to hold data from textbox
            temp = Encoding.ASCII.GetBytes(txtEepromWrite.Text);                                ///< Convert string to byte array
            Buffer.BlockCopy(i2cEepromDataAddress, 0, i2cEepromWrite, 0, eepromAddressWriteOffset);///< Copy eeprom starting address 0x00 in starting of buffer
            Buffer.BlockCopy(temp, 0, i2cEepromWrite, eepromAddressWriteOffset, (int)bufferLenght);///< Concatenate data from textbox to buffer
            try
            {
                i2c.I2c_SetConfigInt(i2cHandle, "SlaveAddr", i2cEepromAddress, TdxCommon.ParamStorageType.StoreVolatile);
                unsafe
                {
                    fixed (byte* for_Casting_Intptr_to_Byte = i2cEepromWrite)
                    {
                        returnValue = i2c.I2c_Write(i2cHandle, (IntPtr)for_Casting_Intptr_to_Byte, bufferLenght + eepromAddressWriteOffset); ///< write buffer to EEPROM starting address 0X00
                        if(returnValue == 0)
                            throw new Exception();                                            ///< If Write operation returns 0
                    }
                }
            }
            catch
            {
                lblMessage.Text = "Write failed";
            }
        }

        /// <summary>
        /// Read from EEPROM from starting addres i.e 0x00
        /// Note: Write operation before Read operation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EepromRead(object sender, EventArgs e)
        {
            btnRTCRead.Enabled = true;                                                          ///< GUI code
            rdbtnAuto.Checked = false;
            tmrRTCRead.Enabled = false;
            rdbtnManual.Checked = true;
            lblMessage.Text = " ";

            Byte[] i2cEepromRead = new Byte[bufferLenght];
            try
            {
                i2c.I2c_SetConfigInt(i2cHandle, "SlaveAddr", i2cEepromAddress, TdxCommon.ParamStorageType.StoreVolatile);
                unsafe
                {
                    fixed (byte* for_Casting_Intptr_to_Byte = i2cEepromDataAddress)
                    {
                        returnValue = i2c.I2c_Write(i2cHandle, (IntPtr)for_Casting_Intptr_to_Byte, eepromAddressWriteOffset);  ///< Write starting address
                        if(returnValue == 0)
                        throw new Exception();                                                  ///< If Write operation returns 0
                    }
                    System.Threading.Thread.Sleep(10);
                    fixed (byte* for_Casting_Intptr_to_Byte = i2cEepromRead)
                    {
                        returnValue = i2c.I2c_Read(i2cHandle, (IntPtr)for_Casting_Intptr_to_Byte, bufferLenght);///< Read data written above from EEPROM
                        if(returnValue == 0)
                        throw new Exception();                                                  ///< If Read operation returns 0
                    }
                }
            }
            catch
            {
                lblMessage.Text = "Read failed";
            }
            txtEepromRead.Text = Encoding.UTF8.GetString(i2cEepromRead, 0, (int)bufferLenght);  ///< Convert Read byte from I2cRead function to string 
        }
        /// <summary>
        /// Read and Display Date, Time and Day from RTC chip
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RtcReadOperation(object sender, EventArgs e)
        {
            Byte[] i2cDataRead = new Byte[8];                                                   ///< Buffer from where data will be read
            Byte[] tempData = new Byte[13];
            short Seconds = 0;
            short Minutes = 0;
            short Hours = 0;
            short Day = 0;
            short Month = 0;
            int Year = 0;

            i2c.I2c_SetConfigInt(i2cHandle, "SlaveAddr", i2cRTCAddress, TdxCommon.ParamStorageType.StoreVolatile);
            unsafe
            {
                fixed (byte* for_Casting_Intptr_to_Byte = i2cDataRead)
                {
                    returnValue = i2c.I2c_Read(i2cHandle, (IntPtr)for_Casting_Intptr_to_Byte, 8);///< Read 8 frame from RTC chip
                }
            }

            ///<Obtained RTC data is in BCD format  
            tempData[0] = Convert.ToByte(i2cDataRead[0] & 0x0F);              ///< Seconds 
            tempData[1] = Convert.ToByte(i2cDataRead[0] & 0x70);
            tempData[1] >>= 4;

            tempData[2] = Convert.ToByte(i2cDataRead[1] & 0x0F);              ///< Minutes
            tempData[3] = Convert.ToByte(i2cDataRead[1] & 0x70);
            tempData[3] >>= 4;

            tempData[4] = Convert.ToByte(i2cDataRead[2] & 0x0F);              ///< Hours 
            tempData[5] = Convert.ToByte(i2cDataRead[2] & 0x30);
            tempData[5] >>= 4;

            tempData[6] = Convert.ToByte(i2cDataRead[3] & 0x07);              ///< Day

            tempData[7] = Convert.ToByte(i2cDataRead[4] & 0x0F);              ///< Date
            tempData[8] = Convert.ToByte(i2cDataRead[4] & 0x30);
            tempData[8] >>= 4;

            tempData[9] = Convert.ToByte(i2cDataRead[5] & 0x0F);              ///< Month
            tempData[10] = Convert.ToByte(i2cDataRead[5] & 0x10);
            tempData[10] >>= 4;

            tempData[11] = Convert.ToByte(i2cDataRead[6] & 0x0F);             ///< Year
            tempData[12] = Convert.ToByte(i2cDataRead[6] & 0xF0);
            tempData[12] >>= 4;

            Seconds = Convert.ToInt16(String.Concat(tempData[1], tempData[0]));
            Minutes = Convert.ToInt16(String.Concat(tempData[3], tempData[2]));
            Hours = Convert.ToInt16(String.Concat(tempData[5], tempData[4]));
            Day = Convert.ToInt16(String.Concat(tempData[8], tempData[7]));
            Month = Convert.ToInt16(String.Concat(tempData[10], tempData[9]));
            Year = Convert.ToInt16(String.Concat(tempData[12], tempData[11])) + 2000;

            DateTime dt = new DateTime(Year, Month, Day, Hours, Minutes, Seconds);
            txtRTCRead.Text = "Time (HH:MM:SS) : " + dt.ToLongTimeString() + Environment.NewLine +
                              "Date (DD/MM/YY)  : " + dt.ToShortDateString() + Environment.NewLine +
                              "Day is                   : " + dt.DayOfWeek.ToString();
        }
        /// <summary>
        /// Form close event: Deint I2C handle and close I2C library
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (i2cHandle != IntPtr.Zero)
            {
                i2c.I2c_Close(i2cHandle);                                           ///< Close I2C library
                i2c.I2c_Deinit(i2cHandle);                                          ///< Deinit I2C Handle
            }
        }
    }
}